handlerGroups = { # The first group that takes control of a process has priority
	"browser": {
		"idents": [
			"binSimple:firefox",
			"binSimple:firefox-esr",
			"binSimple:palemoon",
			"binSimple:python*;cmdlineSimple:*gawk.py",
			"binSimple:WebKitWebProcess"
		],
		"treatAllMatchedAsFocused": True,
		"propsFocused": {
			"memory.high": "256M"
		},
		"propsUnfocused": {
			"memory.high": "256M",
			"cpu.weight": "1"
		}
	},
	"telegram": {
		"idents": ["binSimple:telegram-desktop"],
		"propsFocused": {
			"memory.high": "64M"
		},
		"propsUnfocused": {
			"memory.high": "64M",
			"cpu.weight": "10"
		}
	},
	"chocolate-doom": {
		"idents": ["binSimple:chocolate-doom"],
		"propsFocused": {
		},
		"propsUnfocused": {
			"cpu.weight": "10"
		}
	},
	"firealpaca": {
		"idents": ["binSimple:wine"],
		"propsFocused": {
			"memory.high": "256M"
		},
		"propsUnfocused": {
			"memory.high": "256M",
			"cpu.weight": "1"
		}
	},
	"vlc": {
		"idents": ["binSimple:vlc"],
		"propsFocused": {
		},
		"propsUnfocused": {
			"cpu.weight": "25"
		}
	},
	"low-priority": {
		"idents": [
			"binSimple:python*;cmdlineSimple:*zram-collector",
			"binSimple:python*;cmdlineSimple:*window-cg",
		],
		"propsFocused": {
			"cpu.weight": "1"
		},
		"propsUnfocused": {
			"cpu.weight": "1"
		}
	},
	"high-priority": {
		"idents": ["binSimple:Xorg","binSimple:pulseaudio"],
		"propsFocused": {
			"cpu.weight": "1000"
		},
		"propsUnfocused": {
			"cpu.weight": "1000"
		}
	},
	"all": {
		"idents": ["binSimple:*"],
		"propsFocused": {
			"cpu.weight": "100"
		},
		"propsUnfocused": {
			"cpu.weight": "10"
		}
	}
}

import fnmatch

for group in handlerGroups:
	if not "propsFocused" in handlerGroups[group]: handlerGroups[group]["propsFocused"] = {}
	if not "propsUnfocused" in handlerGroups[group]: handlerGroups[group]["propsUnfocused"] = {}

requiredControllers = []
for group in handlerGroups: # Figure out the required controllers
	for ptype in ["propsFocused","propsUnfocused"]:
		for controller in handlerGroups[group][ptype]:
			controller = controller.split(".",1)[0]
			if not controller in requiredControllers:
				requiredControllers.append(controller)
_g.cgroup.addControllers("",*requiredControllers)
_g.cgroup.create("window-cg")
_g.cgroup.addControllers("window-cg",*requiredControllers)

# Create cgroups
for group in handlerGroups:
	_g.cgroup.create("window-cg/" +group+ ".focused")
	_g.cgroup.create("window-cg/" +group+ ".unfocused")
	for key in handlerGroups[group]["propsFocused"]:
		_g.cgroup.setProp("window-cg/" +group+ ".focused",key,handlerGroups[group]["propsFocused"][key])
	for key in handlerGroups[group]["propsUnfocused"]:
		_g.cgroup.setProp("window-cg/" +group+ ".unfocused",key,handlerGroups[group]["propsUnfocused"][key])

def doesProcMatch(proc,group):
	for identGroup in group["idents"]:
		matched = True
		for ident in identGroup.split(";"):
			iSplit = ident.split(":",1)
			if not fnmatch.fnmatch(proc[iSplit[0]],iSplit[1]):
				matched = False
				break
		if matched == True:
			return True
	return False

def windowChanged(window):
	pid = _g.windowSystem.getWindowPID(window)
	procs = _g.proc.getList()
	focusedGroup = None
	
	if pid in procs:
		print("Process: " +str(procs[pid]))
	
	for group in handlerGroups:
		try:
			if doesProcMatch(procs[pid],handlerGroups[group]):
				focusedGroup = group
				break
		except Exception:
			pass
	
	print("Focused group: " +str(focusedGroup))
	
	for group in handlerGroups:
		unhook = handlerGroups[group].get("unhookUnmatchedChildren",True)
		if unhook:
			groupProcs = _g.cgroup.getProcs("window-cg/" + group + ".focused") + _g.cgroup.getProcs("window-cg/" + group + ".unfocused")
		
		for proc in list(procs):
			match = False
			try:
				match = doesProcMatch(procs[proc],handlerGroups[group])
			except Exception:
				pass
			
			if unhook:
				try:
					if not match and proc in groupProcs:
						_g.cgroup.setGroup(proc,"")
				except Exception:
					pass
			
			try:
				if not match: continue
				pproc = procs[proc]
				del procs[proc]
				
				if focusedGroup == group:
					if handlerGroups[group].get("treatAllMatchedAsFocused",False):
						_g.cgroup.setGroup(pproc["pid"],"window-cg/" + group + ".focused")
					elif pproc["pid"] == pid:
						_g.cgroup.setGroup(pproc["pid"],"window-cg/" + group + ".focused")
				else:
					_g.cgroup.setGroup(pproc["pid"],"window-cg/" + group + ".unfocused")
			except Exception:
				pass
